/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre;

import java.io.File;
import java.util.LinkedList;

import org.apache.commons.configuration.HierarchicalConfiguration;
import org.apache.commons.configuration.XMLConfiguration;

public class Config {
    public final static String CONFIG_DIR = ".jarre" + File.separator;
    public final static XMLConfiguration xmlConfig = new XMLConfiguration(new HierarchicalConfiguration()) {
	private static final long serialVersionUID = 1L;
	@Override public void setProperty(String arg0, Object arg1) {
	    if (arg1 instanceof String) {
		String s = (String)arg1;
		StringBuilder builder = new StringBuilder();
		for (char c: s.toCharArray())
		    switch (c) {
			case ',':
			    builder.append("\\" + getDefaultListDelimiter());
			    break;
			default:
			    builder.append(c);
		    }
		super.setProperty(arg0, builder.toString());
	    } else super.setProperty(arg0, arg1);
	}
    };
    private static String xmlConfigFileName;
    public static void create() {
	String home = System.getProperty("user.home");
	File confDir = new File(home + File.separator + CONFIG_DIR);
	if (! confDir.exists()) confDir.mkdir();
	String configDir = confDir.getPath() + File.separator;
	xmlConfigFileName = configDir + "config.xml";
	try {
	    xmlConfig.load(xmlConfigFileName);
	} catch (Exception dontExists) {
	    try {
		xmlConfig.setEncoding("UTF-8");
		xmlConfig.setRootElementName("jarre-config");
		xmlConfig.setProperty("config[@version]", "0.1");
		xmlConfig.save(xmlConfigFileName);
	    } catch (Exception cantCreate) {
		System.out.println("Nie można utworzyć pliku konfiguracyjnego.");
	    }
	}
	xmlConfig.setDelimiterParsingDisabled(false);
    }
    public static void setStringArray(String path, String[] array) {
	int i = 0;
	for (String s: array)
	    Config.xmlConfig.setProperty(path + ".entry_" + i++, s);
    }
    public static String[] getPropertyArray(String path) {
	LinkedList<String> list = new LinkedList<String>();
	int i = 1;
	String s = Config.xmlConfig.getString(path + ".entry_0", null);
	while (s != null) {
	    list.add(s);
	    s = Config.xmlConfig.getString(path + ".entry_" + i++, null);
	}
	return list.toArray(new String[0]);
    }
    public static void close() {
	try {
	    xmlConfig.save(xmlConfigFileName);
	} catch (Exception e) {
	    System.out.println("Nie można utworzyć pliku konfiguracyjnego.");
	}
    }
}
