/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre;

import static jarre.i18n.I18n._;
import jarre.app.MainWindow;
import jarre.midi.Device;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

public class Jarre {
    public static Color[] colors;
    public static MainWindow window;
    public static void main(String[] args) {
	Display display = new Display();
	try {
	    Config.create();
	    searchForMidiDevices();
	    colors = new Color[] {
		createColor(0xADD82F),
		createColor(0xA020F0),
		createColor(0x90EE90),
		createColor(0xFFC0CB),
		createColor(0xFFA500),
		createColor(0x1E90FF),
		createColor(0xFF0000),
		createColor(0x800000),
		createColor(0xFFFF00),
		createColor(0x808000),
		createColor(0xFF00FF),
		createColor(0x800080),
		createColor(0x00FF00),
		createColor(0x008000),
		createColor(0x00FFFF),
		createColor(0x008080),
		createColor(0x0000FF),
		createColor(0x000080)
	    };
	    Shell shell = new Shell(display);
	    window = new MainWindow(shell, "Jarre");
	    window.open();
	} catch (Exception e) {
	    e.printStackTrace();
	} finally {
	    for (Color c: colors) c.dispose();
	    if (display.isDisposed()) display.dispose();
	    Config.close();
	}
    }
    public static Color createColor(int color) {
	return new Color(Display.getDefault(), (color & 0xFF0000) >> 16, (color & 0x00FF00) >> 8, color & 0x0000FF);
    }
    private static void searchForMidiDevices() {
	final Shell shell = new Shell();
	final Display display = shell.getDisplay();
	shell.setLayout(new GridLayout(2, false));
	shell.addListener(SWT.Close, new Listener() {
	    @Override public void handleEvent(Event event) {
		event.doit = false;
	    }
	});
	Label label = new Label(shell, SWT.NONE);
	label.setImage(display.getSystemImage(SWT.ICON_WORKING));
	label = new Label(shell, SWT.NONE);
	label.setText(_("Please wait. Searching for midi devices."));
	Thread thread = new Thread() {
	    @Override public void run() {
		Device.initialize();
		display.syncExec(new Runnable() {
		    @Override public void run() {
			shell.dispose();
		    }
		});
	    }
	};
	thread.start();
	shell.pack();
	shell.open();
	while (!shell.isDisposed()) {
	    if (!display.readAndDispatch()) display.sleep();
	}
    }
}
