/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.app;

import static jarre.i18n.I18n._;
import jarre.Config;
import jarre.midi.Device;
import jarre.widgets.common.AppWindow;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import javax.sound.midi.MidiDevice;
import javax.sound.midi.MidiDevice.Info;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;

public class ConfigWindow extends AppWindow {
    public ConfigWindow(Shell shell, String title) {
	super(new Shell(shell, SWT.APPLICATION_MODAL | SWT.BORDER |  SWT.CLOSE | SWT.DIALOG_TRIM), title);
	getShell().pack();
    }
    @Override public void createControl(Composite parent) {
	String inName = Config.xmlConfig.getString("config.device.in", null);
	String outName = Config.xmlConfig.getString("config.device.out", null);
	parent.setLayout(new GridLayout(2, false));
	new Label(parent, SWT.NONE).setText(_("MIDI input device: "));
	final Button inButton = new Button(parent, SWT.PUSH);
	createMenuForButton(inButton, inName, Device.getInDevices());
	new Label(parent, SWT.NONE).setText(_("MIDI output device: "));
	final Button outButton = new Button(parent, SWT.PUSH);
	createMenuForButton(outButton, outName, Device.getOutDevices());
	Button button = new Button(parent, SWT.CHECK);
	button.setText(_("Connect input to output."));
	button.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, true, 2, 1));
	button.setSelection(Config.xmlConfig.getBoolean("config.device[@redirect]", false));
	button.addListener(SWT.Selection, new Listener() {
	    @Override public void handleEvent(Event event) {
		Button button = (Button)event.widget;
		Config.xmlConfig.setProperty("config.device[@redirect]", button.getSelection());
	    } 
	});
	new Label(parent, SWT.NONE);
	button = new Button(parent, SWT.PUSH);
	button.setText("     " + "OK" + "     ");
	button.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false));
	button.addListener(SWT.Selection, new Listener() {
	    @Override public void handleEvent(Event event) {
		Config.xmlConfig.setProperty("config.device.in", inButton.getText());
		Config.xmlConfig.setProperty("config.device.out", outButton.getText());
		close(SWT.OK);
	    } 
	});
    }
    private void createMenuForButton(final Button button, String inName, List<MidiDevice> devices) {
	final HashMap<String, LinkedList<Info>> infoMap = new HashMap<String, LinkedList<Info>>(); 
	for (MidiDevice d: devices) {
	    Info info = d.getDeviceInfo();
	    String desc = info.getDescription();
	    LinkedList<Info> infoList = infoMap.get(desc);
	    if (infoList == null) {
		infoList = new LinkedList<Info>();
		infoMap.put(desc, infoList);
	    }
	    infoList.add(info);
	    if (inName != null)
		if (inName.equals(info.getName())) button.setText(info.getName());
	}
	button.addListener(SWT.Selection, new Listener() {
	    Menu menu = createMenu(button, infoMap);
	    @Override public void handleEvent(Event event) {
		Point pos = getDisplay().map(button, null, event.x, event.y);
		menu.setLocation(pos);
		menu.setVisible(true);
	    } 
	});
	infoMap.clear();
    }
    private Menu createMenu(Button button, HashMap<String, LinkedList<Info>> infoMap) {
	Menu menu = new Menu(getShell(), SWT.POP_UP);
	if (infoMap.size() == 1) createMenuItems(button, menu, infoMap.values().iterator().next());
	else {
	    for (String s: infoMap.keySet()) {
		MenuItem item = new MenuItem(menu, SWT.CASCADE);
		item.setText(s);
		Menu subMenu = new Menu(getShell(), SWT.DROP_DOWN);
		item.setMenu(subMenu);
		createMenuItems(button, subMenu, infoMap.get(s));
	    }
	}
	return menu;
    }
    private void createMenuItems(final Button button, Menu menu, LinkedList<Info> infoList) {
	for (Info i: infoList) {
	    MenuItem item = new MenuItem(menu, SWT.PUSH);
	    final String name = i.getName();
	    item.setText(name);
	    item.addListener(SWT.Selection, new Listener() {
		@Override public void handleEvent(Event event) {
		    button.setText(name);
		    button.pack(true);
		    button.redraw();
		    button.update();
		    getShell().pack(true);
		    getShell().redraw();
		    getShell().update();
		}
	    });
	}
    }
}
