/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.keyboard;

import org.eclipse.swt.graphics.Point;

public class KeyboardPainterManager {
    private static int leftNote = roundNoteDown(21);
    private static int rightNote = roundNoteUp(108);
    private static int width = 676;
    private static int height = 84;
    private static IKeyboardPainter currentPainter;
    public static IKeyboardPainter getCurrentPainter() {
	if (currentPainter == null) {
	    currentPainter = new SimpleKeyboardPainter();
	    setKeyRange(leftNote, rightNote);
	    setKeyboardSize(width, height);
	}
	return currentPainter;
    }
    public static void setKeyRange(int leftNote, int rightNote) {
	KeyboardPainterManager.leftNote = roundNoteDown(leftNote);
	KeyboardPainterManager.rightNote = roundNoteUp(rightNote);
	if (currentPainter != null) {
	    currentPainter.setKeyRange(KeyboardPainterManager.leftNote, KeyboardPainterManager.rightNote);
	    setKeyboardSize(width, height);
	}
    }
    public static void setKeyboardSize(int width, int height) {
	if (currentPainter != null) {
	    Point size = currentPainter.computeSize(width, height);
	    width = size.x;
	    height = size.y;
	}
	KeyboardPainterManager.width = width;
	KeyboardPainterManager.height = height;
	if (currentPainter != null) currentPainter.setKeyboardSize(width, height);
    }
    public static int getLeftNote() {
	return leftNote;
    }
    public static int getRightNote() {
	return rightNote;
    }
    public static int getWidth() {
	return width;
    }
    public static int getHeight() {
	return height;
    }
    public static int roundNoteDown(int note) {
	switch (note % 12) {
	    case 1:
	    case 3:
	    case 6:
	    case 8:
	    case 10:
		return note - 1;
	    default:
		return note;
	}
    }
    public static int roundNoteUp(int note) {
	switch (note % 12) {
	    case 1:
	    case 3:
	    case 6:
	    case 8:
	    case 10:
		return note + 1;
	    default:
		return note;
	}	
    }
    public static int countWhiteKeys(int leftNote, int rightNote) {
	int whiteKeyCount = 0;
	for (int i = leftNote; i <= rightNote; i++) {
	    KeyType type = getKeyType(i);
	    if (type.isWhiteKey) whiteKeyCount++;
	}
	return whiteKeyCount;
    }
    public static KeyType getKeyType(int note) {
	return KeyType.values()[note % 12];
    }
}
