/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.midi;

import java.util.ArrayList;
import java.util.List;

import javax.sound.midi.MidiDevice;
import javax.sound.midi.MidiSystem;
import javax.sound.midi.MidiUnavailableException;
import javax.sound.midi.Sequencer;
import javax.sound.midi.Synthesizer;

public class Device {
    private static List<MidiDevice> inputDevices = new ArrayList<MidiDevice>();;
    private static List<MidiDevice> outputDevices = new ArrayList<MidiDevice>();;
    public static void initialize() {
	MidiDevice.Info[] info = MidiSystem.getMidiDeviceInfo();
	for (MidiDevice.Info i: info) {
	    MidiDevice device = null;
	    try {
		device = MidiSystem.getMidiDevice(i);
	    } catch (MidiUnavailableException e) { }
	    if (!(device instanceof Synthesizer) && !(device instanceof Sequencer))
	    {
		if (device.getMaxTransmitters() != 0) inputDevices.add(device);
		if (device.getMaxReceivers() != 0) outputDevices.add(device);
	    }
	}
    }
    public static List<MidiDevice> getInDevices() {
	return inputDevices;
    }
    public static List<MidiDevice> getOutDevices() {
	return outputDevices;
    }
    public static MidiDevice getInputByName(String name) {
	return getDeviceByName(inputDevices, name);
    }
    public static MidiDevice getOutputByName(String name) {
	return getDeviceByName(outputDevices, name);
    }
    private static MidiDevice getDeviceByName(List<MidiDevice> list, String name) {
	for (MidiDevice dev: list)
	    if (dev.getDeviceInfo().getName().equals(name)) return dev;
	return null;	
    }
}
