/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.midi.event;

import javax.sound.midi.MetaMessage;
import javax.sound.midi.ShortMessage;
import javax.sound.midi.SysexMessage;

public class EventMessage {
    public final EventEnum type;
    public final Object event;
    public EventMessage(NoteOnEvent event) {
	this.type = EventEnum.NOTE_ON;
	this.event = event;
    }
    public EventMessage(NoteOffEvent event) {
	this.type = EventEnum.NOTE_OFF;
	this.event = event;
    }
    public EventMessage(ProgramChangeEvent event) {
	this.type = EventEnum.PROGRAM_CHANGE;
	this.event = event;
    }
    public EventMessage(TempoChangeEvent event) {
	this.type = EventEnum.TEMPO_CHANGE;
	this.event = event;
    }
    public EventMessage(TimeSignatureEvent event) {
	this.type = EventEnum.TIME_SIGNATURE;
	this.event = event;
    }
    public EventMessage(ShortMessage shortMessage) {
	this.type = EventEnum.OTHER_SHORT_MESSAGE;
	this.event = shortMessage;
    }
    public EventMessage(MetaMessage metaMessage) {
	this.type = EventEnum.OTHER_META_MESSAGE;
	this.event = metaMessage;
    }
    public EventMessage(SysexMessage sysexMessage) {
	this.type = EventEnum.OTHER_SYSEX_MESSAGE;
	this.event = sysexMessage;
    }
    public EventMessage(EventEnum type) {
	this.type = type;
	this.event = null;
    }
    @SuppressWarnings("unchecked")
    public <U> U getEvent() {
	return (U)event;
    }
    @Override public String toString() {
	return type + " " + event;
    }
}
