/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.midi.event;

import javax.sound.midi.MetaMessage;

public class TimeSignatureEvent extends NoteEvent {
    public final int numerator;
    public final int denumerator;
    public final int previousNumerator;
    public final int previousDenumerator;
    public TimeSignatureEvent(MetaMessage metaMessage, MetaMessage previousMetaMessage, int trackNum, int tick) {
	super(trackNum, tick);
	byte[] b = metaMessage.getData();
	numerator = b[0];
	denumerator = 1 << b[1];
	if (previousMetaMessage == null) {
	    previousNumerator = 4;
	    previousDenumerator = 4;
	} else {
	    b = previousMetaMessage.getData();
	    previousNumerator = b[0];
	    previousDenumerator = 1 << b[1];
	}
    }
    public TimeSignatureEvent(int numerator, int denumerator) {
	super(-1, -1);
	this.numerator = numerator;
	this.denumerator = denumerator;
	this.previousNumerator = 4;
	this.previousDenumerator = 4;
    }
    @Override public String toString() {
	return "TimeSignatureEvent " + numerator + ", " + denumerator;
    }
}
