/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.midi.sequencer;

import java.util.concurrent.TimeUnit;

public abstract class SeqTimer {
    private volatile boolean running = false;
    private final int period;
    private Thread thread;
    public SeqTimer(int period) {
	this.period = period;
    }
    public void start() {
	if (!running) {
	    running = true;
	    thread = new Thread() {
		@Override public void run() {
		    long time, oldTime;
		    oldTime = time = System.currentTimeMillis();
		    while (process((int)(time - oldTime))) {
			try {
			    TimeUnit.MILLISECONDS.sleep(period);
			} catch (InterruptedException e) {
			}
			oldTime = time;
			time = System.currentTimeMillis();
		    }
		    running = false;
		}
	    };
	    init();
	    thread.start();
	}
    }
    public void join() throws InterruptedException {
	if (thread != null) thread.join();
    }
    public boolean isRunning() {
	return running;
    }
    public void init() {
    }
    public abstract boolean process(int deltaTime);
}
