/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.midi.sequencer;

import java.util.concurrent.atomic.AtomicInteger;

import jarre.midi.MySequence;
import jarre.midi.event.TimeSignatureEvent;

import javax.sound.midi.InvalidMidiDataException;
import javax.sound.midi.MidiMessage;
import javax.sound.midi.Receiver;
import javax.sound.midi.ShortMessage;

public abstract class SequencerAbstract extends SequencerBase {
    private final static ShortMessage metronomeMessage = new ShortMessage();
    static {
	try {
	    metronomeMessage.setMessage(ShortMessage.NOTE_ON, 9, 37, 127);
	} catch (InvalidMidiDataException e1) {
	    System.out.println("Never should be here. SequencerAbstract line 35");
	    e1.printStackTrace();
	}
    }
    private Receiver receiver;
    private AtomicInteger speed = new AtomicInteger(100);
    protected boolean metronome = false;
    protected int tempo;
    protected boolean ignoreProgramChange = false;
    protected volatile boolean followMode = false;
    protected int numerator = 4;
    protected int denumerator = 4;
    protected MySequence sequence;
    public void allSoundOff() {
	for (int i = 0; i < 16; i++) channelSoundOff(i);
    }
    public void channelSoundOff(int channel) {
	try {
	    ShortMessage message = new ShortMessage();
	    message.setMessage(ShortMessage.CONTROL_CHANGE, channel, 0x7B /* All notes off */, 0);
	    dispatchMessage(message, -1);
	    message = new ShortMessage();
	    message.setMessage(ShortMessage.CONTROL_CHANGE, channel, 0x40 /* Sustain off*/ , 0);
	    dispatchMessage(message, -1);
	} catch (InvalidMidiDataException e) {
	    System.out.println("Never should be here. SequencerAbstract line 52");
	    e.printStackTrace();
	}
    }
    public void dispatchMessage(MidiMessage message, int tick) {
	if (receiver != null) receiver.send(message, tick);
    }
    public void dispatchMetronomeMessage() {
	dispatchMessage(metronomeMessage, -1);
    }
    public int getDenumerator() {
	return denumerator;
    }
    public int getNumerator() {
	return numerator;
    }
    public MySequence getSequence() {
	return sequence;
    }
    public void setReceiver(Receiver receiver) {
	this.receiver = receiver;
    }
    public void setSequence(MySequence sequence) {
	this.sequence = sequence;
	setTempo(500000);
	setTimeSignature(sequence.getTimeSignature());
    	sendSequenceChanged(sequence);
    }
    public void setFollowMode(boolean followMode) {
	this.followMode = followMode;
    }
    public boolean getFollowMode() {
	return followMode;
    }
    public boolean getIgnoreProgramChange() {
        return ignoreProgramChange;
    }
    public void setIgnoreProgramChange(boolean ignoreProgramChange) {
        this.ignoreProgramChange = ignoreProgramChange;
    }
    public boolean getMetronome() {
        return metronome;
    }
    public void setMetronome(boolean metronome) {
        this.metronome = metronome;
    }
    public void setSpeed(int speed) {
	this.speed.getAndSet(speed);
    }
    public int getSpeed() {
	return speed.getAndAdd(0);
    }
    public void setTempo(int tempo) {
	this.tempo = tempo / sequence.getResolution();
    }
    public void setTimeSignature(int numerator, int denumerator) {
	this.numerator = numerator;
	this.denumerator = denumerator;
    }
    public synchronized void setTimeSignature(TimeSignatureEvent timeSignature) {
	setTimeSignature(timeSignature.numerator, timeSignature.denumerator);
    }
    public void goForward() {
	if (sequence == null) return;
	goForward(2 * sequence.getResolution());
    }
    public void goBackward() {
	if (sequence == null) return;
	goBackward(2 * sequence.getResolution());
    }
    public void goToStart() {
	goToPosition(0);
    }
    public abstract void play();
    public abstract void pause();
    public abstract boolean isPlaying();
    public abstract int getCurrentPos();
    public abstract void noteChangeSignal();
    public abstract boolean isWaiting();
    public abstract void goForward(int ticks);
    public abstract void goBackward(int ticks);
    public abstract void goToPosition(int seconds);
}
