/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.midi.sequencer;

import jarre.midi.MySequence;
import jarre.midi.event.NoteOffEvent;
import jarre.midi.event.NoteOnEvent;

import java.util.ArrayList;
import java.util.List;

public abstract class SequencerBase {
    private final List<SeqNoteEvent> noteEventList = new ArrayList<SeqNoteEvent>();
    private final List<SeqDrawEvent> drawEventList = new ArrayList<SeqDrawEvent>();
    private final List<SeqTrackEvent> trackEventList = new ArrayList<SeqTrackEvent>();
    public void addNoteEvent(SeqNoteEvent noteEvent) {
	synchronized (noteEventList) {
	    noteEventList.add(noteEvent);
	}
    }
    public void addDrawEvent(SeqDrawEvent drawEvent) {
	synchronized (drawEventList) {
	    drawEventList.add(drawEvent);
	}
    }
    public void addTrackEvent(SeqTrackEvent trackEvent) {
	synchronized (trackEventList) {
	    trackEventList.add(trackEvent);
	}
    }
    protected void sendNoteOnEvent(NoteOnEvent noteOn) {
	synchronized (noteEventList) {
	    for (SeqNoteEvent noteEvent: noteEventList) noteEvent.noteOnEvent(noteOn);
	}
    }
    protected void sendNoteOffEvent(NoteOffEvent noteOff) {
	synchronized (noteEventList) {
	    for (SeqNoteEvent noteEvent: noteEventList) noteEvent.noteOffEvent(noteOff);
	}
    }
    protected void sendAllNotesOffEvent() {
	synchronized (noteEventList) {
	    for (SeqNoteEvent noteEvent: noteEventList) noteEvent.allNotesOffEvent();
	}
    }
    protected void sendDrawEvent(int currentPos) {
	synchronized (drawEventList) {
	    for (SeqDrawEvent drawEvent: drawEventList) drawEvent.drawSequence(currentPos);
	}
    }
    protected void sendMoveEvent(int currentPos) {
	synchronized (drawEventList) {
	    for (SeqDrawEvent drawEvent: drawEventList) drawEvent.moveSequence(currentPos);
	}
    }
    protected void sendSequenceChanged(MySequence sequence) {
	synchronized (trackEventList) {
	    for (SeqTrackEvent trackEvent: trackEventList) trackEvent.sequenceChanged(sequence);
	}
    }
    protected void sendEndOfTracksEvent() {
	synchronized (trackEventList) {
	    for (SeqTrackEvent trackEvent: trackEventList) trackEvent.endOfTracksEvent();
	}
    }
}
