/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.reflect;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.HashMap;

public class Reflection {
    public static <T> T createInstance(Class<T> Interface, Class<?> clazz) throws NoSuchMethodException {
	HashMap<String, Object> map = new HashMap<String, Object>();
	Method[] iMethods = Interface.getDeclaredMethods();
	for (int x = 0; x < iMethods.length; x++) {
	    Method method = iMethods[x];
	    if (method.getName().equals("constructor")) {
		Constructor<?>  c = findConstructor(method.getParameterTypes(), clazz);
		map.put(method.getName(), c);
	    } else {
		Method m = findMethod(method.getParameterTypes(), clazz, method.getName());
		map.put(method.getName(), m);
	    }
	}
	return Proxy.newInstance(map, Interface);
    }
    private static Constructor<?> findConstructor(Class<?>[] parameters, Class<?> clazz) throws NoSuchMethodException {
	for (Constructor<?> c: clazz.getConstructors()) {
	    Class<?>[] cParams = c.getParameterTypes();
	    if (cParams.length == parameters.length) {
		boolean match = true;
		for (int x = 0; x < cParams.length; x++)
		    if (! parameters[x].isAssignableFrom(cParams[x])) match = false;
		if (match) return c;
	    }
	}
	throw new NoSuchMethodException(clazz.getName() + "(" + Arrays.toString(parameters) + ")");
    }
    private static Method findMethod(Class<?>[] parameters, Class<?> clazz, String name) throws NoSuchMethodException {
	Class<?> c = clazz;
	for (Method m: c.getMethods())
	    if (m.getName().equals(name) && Arrays.equals(m.getParameterTypes(), parameters)) return m;
	while ((c = c.getSuperclass()) != null) {
	    for (Method m: c.getMethods())
		if (m.getName().equals(name) && Arrays.equals(m.getParameterTypes(), parameters)) return m;
	}
	throw new NoSuchMethodException(clazz.getName() + "." + name + "(" + Arrays.toString(parameters) + ")");
    }
}
