/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.widgets.common;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;

public class ColorWidget extends BasicControl {
    private Color color;
    private Color black;
    private int border = 5;
    public ColorWidget(Composite parent, int style) {
	super(parent, style);
	black = getDisplay().getSystemColor(SWT.COLOR_BLACK);
    }
    public void setColor(int r, int g, int b) {
	if (color != null) color.dispose();
	color = new Color(getDisplay(), r, g, b);
    }
    public void setColor(Color color) {
	this.color = new Color(getDisplay(), color.getRGB());
    }
    public void setBorder(int border) {
	this.border = border;
    }
    public Color getColor() {
	return color;
    }
    @Override protected void paintControl(PaintEvent e) {
	GC gc = e.gc;
	Point size = getSize();
	gc.setForeground(black);
	gc.setBackground(color);
	gc.drawRectangle(border, border, size.x - 2 * border, size.y - 2 * border);
	gc.fillRectangle(border + 1, border + 1, size.x - 2 * border - 1, size.y - 2 * border - 1);
    }
    @Override public Point computeSize(int wHint, int hHint, boolean changed) {
	if (wHint == SWT.DEFAULT && hHint == SWT.DEFAULT) return new Point(32, 12);
	if (wHint == SWT.DEFAULT) return new Point(32, 12);
	if (hHint == SWT.DEFAULT) return new Point(wHint, 12);
	return new Point(wHint, hHint);
    }
    @Override public void dispose() {
	color.dispose();
	super.dispose();
    }
}
