/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.widgets.common;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;

public class PlayerProgressBar extends Composite implements IProgressBar {
    private Label currentTime;
    private Label songTime;
    private IProgressBar progressBar;
    public PlayerProgressBar(Composite parent, int style) {
	super(parent, style);
	setLayout(new GridLayout(4, false));
	currentTime = new Label(this, SWT.NONE);
	new Label(this, SWT.NONE).setText("/");
	songTime = new Label(this, SWT.NONE);
	if (SWT.getPlatform().equals("win32")) progressBar = new SWTJSlider(this, SWT.NONE);
	else progressBar = new SWTScale(this, SWT.NONE);
	((ILayoutFunctions)progressBar).setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
	progressBar.setPageIncrement(1);
	progressBar.setMinimum(0);
	setCurrentTime(0);
	setSongTime(0);
    }
    public void setCurrentTime(int seconds) {
	currentTime.setText(getString(seconds));
	progressBar.setSelection(seconds);
    }
    public void setSongTime(int seconds) {
	songTime.setText(getString(seconds));
	progressBar.setMaximum(seconds);
    }
    @Override public void addSelectionListener(Listener listener) {
	addListener(SWT.Selection, listener);
    }
    @Override public void addListener(int eventType, final Listener listener) {
	if (eventType == SWT.Selection) {
	    progressBar.addSelectionListener(new Listener() {
		@Override public void handleEvent(Event event) {
		    event.widget = PlayerProgressBar.this;
		    listener.handleEvent(event);
		}
	    });
	} else super.addListener(eventType, listener);
    }
    @Override public int getMaximum() {
	return progressBar.getMaximum();
    }
    @Override public int getMinimum() {
	return progressBar.getMinimum();
    }
    @Override public int getPageIncrement() {
	return progressBar.getPageIncrement();
    }
    @Override public int getSelection() {
	return progressBar.getSelection();
    }
    @Override public void setMaximum(int maximum) {
	progressBar.setMaximum(maximum);
    }
    @Override public void setMinimum(int minimum) {
	progressBar.setMaximum(minimum);
    }
    @Override public void setPageIncrement(int increment) {
	progressBar.setPageIncrement(increment);
    }
    @Override public void setSelection(int value) {
	progressBar.setSelection(value);
    }
    private String getString(int seconds) {
	String m = "" + seconds / 60;
	String s = "" + seconds % 60;
	return m + ":" + ((s.length() > 1) ? s: "0" + s);
    }
}
