/*  Programmer:	Piotr Borek
 *  E-mail: piotrborek@op.pl
 *  Copyright 2009 Piotr Borek
 *
 *  Distributed under the terms of the GPL (GNU Public License)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jarre.widgets.common;

import java.awt.Container;
import java.awt.Frame;

import javax.swing.JApplet;
import javax.swing.JSlider;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.eclipse.swt.SWT;
import org.eclipse.swt.awt.SWT_AWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

public class SWTJSlider extends Composite implements IProgressBar, ILayoutFunctions {
    private Frame frame;
    private JApplet applet;
    private JSlider slider;
    private volatile boolean enableEvents = true;
    public SWTJSlider(Composite parent, int style) {
	super(parent, style | SWT.EMBEDDED | SWT.NO_BACKGROUND);
	RowLayout layout = new RowLayout();
	layout.marginBottom = 12;
	layout.marginTop = 12;
	setLayout(layout);
	setLookAndFeel();
	frame = SWT_AWT.new_Frame(this);
	applet = new JApplet();
	frame.add(applet);
	Container container = applet.getContentPane();
	slider = new JSlider();
	Color color = getBackground();
	slider.setBackground(new java.awt.Color(color.getRed(), color.getGreen(), color.getBlue()));
	container.add(slider);
    }
    @Override public void addSelectionListener(Listener listener) {
	addListener(SWT.Selection, listener);
    }
    @Override public void setSelection(int value) {
	enableEvents = false;
	slider.setValue(value);
    }
    @Override public int getSelection() {
	return slider.getValue();
    }
    @Override public void setMinimum(int minimum) {
	enableEvents = false;
	slider.setMinimum(minimum);
    }
    @Override public int getMinimum() {
	return slider.getMinimum();
    }
    @Override public void setMaximum(int maximum) {
	enableEvents = false;
	slider.setMaximum(maximum);
    }
    @Override public int getMaximum() {
	return slider.getMaximum();
    }
    @Override public void setPageIncrement(int increment) {
	enableEvents = false;
	slider.setMajorTickSpacing(increment);
    }
    @Override public int getPageIncrement() {
	return slider.getMajorTickSpacing();
    }
    @Override public void addListener(int eventType, final Listener listener) {
	if (eventType == SWT.Selection) {
	    slider.addChangeListener(new ChangeListener() {
		@Override public void stateChanged(ChangeEvent event) {
		    if (enableEvents)
			getDisplay().asyncExec(new Runnable() {
			    @Override public void run() {
				Event e = new Event();
				e.widget = SWTJSlider.this;
				listener.handleEvent(e);
			    }
			});
		    else enableEvents = true;
		}
	    });
	} else super.addListener(eventType, listener);
    }
    private static void setLookAndFeel() {
	try {
	    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
	} catch (Exception e) {
	}
    }
}
